/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <math.h>
#include <stdio.h>
#include "startime.h"
#include "angle.h"
#include "sun.h"
#include "moon2.h"
#include "moon3.h"
#include "equationOfTime.h"
#include "tand.h"
#include "eclipse.h"

/*
   jd = julian date for new moon (JD1900 , julian days since 1900)
   ni,nf = new moon integer and fractional parts.
   input: day,month,year 
   [full = 1 full moon] [full = 0, new moon]
   output: date of moon event in JD1900 (julian days since 1900)
 */
extended newmoon(extended day, extended month, extended year, short full, 
		 extended &argumentLatitude,
		 extended &eventDay,
		 extended &eventFraction
		 )
{
  extended j0, dj, k1, k, t, a1, a2,
    f, dd, c, b, e, e1, b1, ni, nf; 

  dj = JulianDays( day, month, year);
  j0 = JulianDays( 0, 1, year);         

  k1 = (year - 1900 + (dj - j0)/365.0 )*12.3685 ;
  if (!full) k = floor(k1+0.5) ;        //new moon
  else       k = floor(k1+0.5) + 0.5 ;  //full moon
  t = k/1236.85;

  a1 = 359.2242 + 360*FRAC(k/1.236886e1) - (3.33e-5 - 3.47e-6*t)*t*t;
  a2 = 306.0253 + 360*FRAC(k/9.330851e-1) + (1.07306e-2*t*t) + (1.236e-5*t*t*t);
  f  = 21.2964 + 360*FRAC(k/9.214926e-1)  - (1.6528e-3 - 2.39e-6*t)*t*t;
  a1 = deg2rad(fmod(a1,360));
  a2 = deg2rad(fmod(a2,360));
  f  = deg2rad(fmod(f,360));
  dd=(1.734e-1 - 3.9e-4*t)*sin(a1)+ 2.1e-3*sin(2*a1)            ;
  dd+= - 4.068e-1*sin(a2) + 1.61e-2*sin(2*a2) - 4e-4*sin(3*a2)           ;
  dd+= + 1.04e-2*sin(2*f) - 5.1e-3*sin(a1+a2)           ;
  dd+= - 7.4e-3*sin(a1-a2) + 4e-4*sin(2*f+a1)           ;
  dd+= - 4e-4*sin(2*f-a1) - 6e-4*sin(2*f+a2) + 1e-3*sin(2*f-a2)           ;
  dd+= + 5e-4*sin(a1+2*a2)           ;

  c  = deg2rad(166.56 + (132.87 - 9.173e-3*t) *t )         ;
  e  = (29.53) * k           ;
  // e1 = rint ( e )           ;
  e1 = rint ( e )           ;
  //  printf("e %lf e1 %lf\n", e, e1);
  b  = dd + e - e1 +  5.8868e-4 * k  + (1.178e-4 - 1.55e-7 * t)*t*t
    + 3.3e-4 * sin(c) + 7.5933e-1           ;

  //  b1 = rint ( b );
  b1 = rint ( b );
  //  printf("b %lf b1 %lf\n", b, b1);
  eventDay = e1 + b1;             // or fi
  eventFraction = b - b1;             // or ff
  argumentLatitude = f;
 return eventDay+eventFraction; // ni + nf;
}

#if 0
/*  calculate new (or full /option full) moon nearest (day,month,year) 
 *  with return in julian time (jd) and latitude (distance from the ecliptic node)
 *  radian units
 */
void newmoonString(char* str, extended day, extended month, extended year, short full, 
		   extended &latitude, julianWithDate &jwd)
{
  extended d, m, y, dow;	

  jwd.jd = newmoon(day, month, year, full, latitude, jwd.nat, jwd.frac);
  CalendarDays( jwd.jd, d, m, y, dow);

  dow = floor(dow);
  if (dow > 7 || dow < 1) dow = 7;
  else dow -= 1;
  m = floor(m);
  if (m > 12 || m < 1) m = 12;
  else m -= 1;
  sprintf(str,"%s \t%s \t%3.0f, %4.0f",
	  dayStringArray[(int)dow], monthStringArray[(int)m], 
	  (float)d, (float)y);
  jwd.dow = dow;
  jwd.day = d;
  jwd.month = m;
  jwd.year = y;
}
#endif

const char *
eclipse_check(extended moon_lat, extended &certain)
{
  moon_lat = fabs(moon_lat-rint(moon_lat/M_PI)*M_PI);
  if (moon_lat > 0.37) moon_lat = M_PI-moon_lat;
  if (moon_lat < 2.42600766e-1){
    certain = 1.0;
    return " certain eclipse";
  }
  else if (moon_lat < 0.37){
    certain = 0.5; 
    return " possible eclipse";
  }
  else {
    certain = 0.0;
    return "";
  }
}

/* Is there a solar eclipse on iday,imonth,iyear?
 * gl,gp,ht = lon,lat,elevation
 * Return 1 if eclipse, 0 otherwise
 * 
 * Based on BASIC routine 7000 ECLIPSE
 * from Peter Duffett-Smith's Astronomy with your Personal Computer
 * Thu Aug  3 18:16:57 PDT 2017
 * My motivation is the July 21, 2017 Solar Eclipse.
 * work-in-progress under construction.
 */ 
int
solar_eclipse(double gl, double gp, double ht, int iday, int imonth, int iyear,
	      double &probabel, double &max_ut_hours)
{
  const int fullmoon = 0;  // 0 signifies new moon
  extended hours_ut, hours_sun_A;
  extended event_day, event_fraction, argument_latitude;
  double ly, my, by, hy, lz, mz, bz, hz, sb;
  double distance_to_moon, phase, solar_longitude, solar_radius;
  double rr, d2, d3, am;
  double sr, hp, zh, tc, xo, sh, s2, z2, ps, z1, rm, rn, hd, ru, rp, pj, r, dd, zd, z6, z7, z9;
  double x0, dm, mr, dp;
  double lj, h0;

  // Check for new moon. 
  newmoon(iday, imonth, iyear, fullmoon, argument_latitude, event_day, event_fraction);

  // Check the argument_latitude for eclipse.
  eclipse_check(argument_latitude, probabel);

  hours_ut = (event_fraction-0.5)*24;
  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;

  //  printf("eclipse probabel %lf hours_ut %lf\n", probabel, hours_ut);
  if (fabs(1.-probabel)>0.1) return 0;

  // Calculate Sun and Moon position -1hr,+1hr event.
  // -1hr  sun ly lon, moon my lon, by lat, hy hor parallax
  // +1hr  sun sb lon diff (sr-ly), mz moon lon, bz moon lat, hz moon hparallax

  double tzi = 0, ds = 0;
  double solar_lon, value_min, value_max, az, el, sun_az, sun_alt;
  double 
    dj00 = floor(event_day),
    v_dj;
  double   v_my, v_by, v_mz, v_bz, hut, v_sr, v_sz;

  hut = hours_ut-1.0;
  //  dj00+=0.5;
  v_dj = JulianDays(iday, imonth, iyear);
  /* char   timestring[255]; */
  /* fullTimeString(timestring, event_day+event_fraction); */
  /* printf("event day %s\n", timestring); */
  /* fullTimeString(timestring, v_dj); */
  /* printf("julian %s\n", timestring); */
  /* fullTimeString(timestring, v_dj+hours_ut/24.0); */
  /* printf("julian %s\n", timestring); */

  /* printf("Start dj %lf (%lf) ut %lf\n", v_dj, event_day, hours_ut); */
  //  printf("dj %lf (%lf)\n", dj00, v_dj);
  dj00 = v_dj;
  //  sun_position(dj00, hut, ly, rr, d2, d3, am);
  calculate_moon(dj00, hut, my, by, hy, distance_to_moon, phase, solar_longitude, solar_radius, true);  // apparent.
  calculate_moon_parallax(dj00, hut, gp, gl, 0, by, my, hy, v_by, v_my);
  printf("ut %lf lat %lf (%lf) lon %lf (%lf) solar lon= %lf\n", hut, v_by, by, v_my, my, solar_longitude);
  //  elliptical_parallax(my, by, v_my, v_by, gl, gp, dj00, hut, hy);
  //  printf("meeus ut %lf lat %lf (%lf) lon %lf (%lf) solar lon= %lf\n", hut, v_by, by, v_my, my, solar_longitude);

  hut = hours_ut+1.0;
  //  sun_position(dj00, hut, lz, rr, d2, d3, am);
  calculate_moon(dj00, hut, mz, bz, hz, distance_to_moon, phase, solar_longitude, solar_radius, true);
  calculate_moon_parallax(dj00, hut, gp, gl, 0, bz, mz, hz, v_bz, v_mz);
  printf("ut %lf lat %lf (%lf) lon %lf (%lf) solar lon= %lf\n", hut, v_bz, bz, v_mz, mz, solar_longitude);
  //  elliptical_parallax(mz, bz, v_mz, v_bz, gl, gp, dj00, hut, hz);
  //  printf("meeus ut %lf lat %lf (%lf) lon %lf (%lf) solar lon= %lf\n", hut, v_bz, bz, v_mz, mz, solar_longitude);

  by=v_by,bz=v_bz,my=v_my,mz=v_mz;

  if (by*bz>0) printf("Error: moon calculation does not cross ecliptic\n");
  
  // Calculate time of lunar latitude = 0, when moon crosses the ecliptic, point A
  // interp lunar latitudes (by, bz) hours per latitude, over two hours
  //  xh = hours_ut;
  x0 = hut - 2 * bz/(bz-by);
  printf("Moon crosses ecliptic at %lf\n", x0);
  // Calculate difference of lunar longitude (-1, +1 hrs)
  dm = mz-my;
  if (dm < 0) dm += 360;
  lj = (dm-sb)/2;

  // Lunar lon at point A (deg)
  mr = my + dm * (x0 - hours_ut+1.)/2.;

  // Time of Sun at point A, account for light speed, in hours ut
  hours_sun_A = x0-1.3851852e-1;

  // Calculate Sun position at point A.
  sun_position(dj00, hours_sun_A, sr, rr, d2, d3, am);
  
#if 0
  //  sun_position();
  sr = sr + dp-5.69e-3; //aberation
  // correct for parallax
  hp = 4.263452e-5/rr;

  //  printf("Sun A %lf lon=%lf dist=%lf d2=%lf d3=%lf am=%lf\n", hours_sun_A, sr, rr, d2, d3, am);
  calculate_moon_parallax(dj00, hours_sun_A, gp, gl, 0, 0, sr, hp, v_sz, v_sr);
  printf("\tsun lon %lf (%lf) lat %lf (%lf)\n", sr, v_sr, 0, v_sz);
  sr = v_sr;
#endif
  // eclipse geometry
  zh = (sr-mr)/lj;
  tc = x0 + zh;
  sh = ((bz-by)*(tc-hours_ut-1)/2.+bz)/lj;
  s2 = sh*sh;
  z2 = zh*zh;
  ps = 4.263e-5/(rr*lj);
  z1 =zh*z2/(z2+s2)+x0;
  //  printf("ECLIPSE TIME MAXIMUM ECLIPSE Z1=%lf\n", z1);
  max_ut_hours = z1;
  h0 =(hy+hz)/(2.*lj);
  //  h0 *= M_PI/180.;
  rm = 2.72446e-1*h0;
  rn = 4.65242e-3/(lj*rr);
  hd = h0*0.99834;
  ru = (hd-rn+ps)*1.02; //umbral radius
  rp = (hd+rn+ps)*1.02; //penumbral radius
  pj = fabs( sh*zh/((s2*z2)*(s2*z2)) );
  r = rm+rn;
  dd = z1-x0;
  dd = dd*dd-(z2-r*r)*dd/zh;
  //  printf("DD=%lf\n", dd);
  zd = dd*dd;
  z6 = z1 - dd;
  //  printf("ECLIPSE TIME FIRST CONTACT Z6=%lf\n", z6);  
    //  if (dd<0) printf("not from this location\n");
  z7 =z1+zd-floor((z1+zd)/24)*24;
  //  printf("ECLIPSE TIME LAST CONTACT Z7=%lf\n", z6);    
  r = rm+ru;
  dd = z1-x0;
  dd= dd*dd-(z2-r*r)*dd/zh;
  z9 = z1+zd-floor((z1+zd)/24)*24;
  //  printf("ECLIPSE TIME LAST CONTACT Z9=%lf\n", z9);    
}

void
calculate_moon_parallax(double dj, double ut, double gp, double gl, double ht, double ecliptic_lat, double ecliptic_lon, 
			double horizontal_parallax, 
			double &v_ecliptic_lat, double &v_ecliptic_lon)
{
  double    
    ra, dec, lha, app_lha, app_dec,lmst;
  //  double  d, m, y, dow, lst;
  geocentric_ecliptic_latlon_to_radec(-1, ra, dec, ecliptic_lon, ecliptic_lat, dj);   // ra in hours
  lmst = local_mean_sidereal_time(dj,ut,gl);
  // these give same answer, 
  //  CalendarDays(dj, d, m, y, dow);
  //  lst = local_sidereal_time(dj, ut, y, gl);
  //  printf("lmst %lf lst %lf\n", lmst, lst);
  lha = lmst - ra/15.0;  // ra from degrees to hours
  compute_parallax(app_lha, app_dec, lha, dec, gl, gp, dj, ut, ht, horizontal_parallax);
  lha = app_lha;
  dec = app_dec;
  ra  = 15.0*(lmst-lha);
  geocentric_ecliptic_latlon_to_radec(+1, ecliptic_lon, ecliptic_lat, ra, dec, dj);   // ra in degrees
  v_ecliptic_lat = ecliptic_lat;
  v_ecliptic_lon = ecliptic_lon;
}

// Meeus Astronomical Algorithms :The Earth's Globe
// lat latitude in degrees
// ht elevation msl in meters
void
topo_earth_ellipsoid_correction(double lat, double ht, double &S, double &C)
{
  double ba = 0.99664719, a = 6378140., b = 6356755.;
  double u;
  //  ba = b/a;
  lat *= M_PI/180.;
  u = ba*tan(lat);
  u = atan(u);
  S = ba*sin(u) + ht/a*sin(lat);
  C = cos(u) + ht/a*cos(lat);
}

#define DEG2RAD(x) ((x)=(M_PI/180.*(x)))
#define RAD2DEG(x) ((x)=(180./M_PI*(x)))

// Meeus Astronomical Algorithms Correction for Parallax
void
elliptical_parallax(double lon, double lat, double &v_lon, double &v_lat,
		    double lst, double gp, double hp, double obliq)
{
  double n;
  double C, S, ht = 0;
  //  double obliq;// lst;  // degrees
  //  obliquity(dj, obliq);
  //  lst = 15*local_mean_sidereal_time(dj, ut, gl);
  //  gp = 33.+21./60.+22./3600.;
  //ht = 1706.;
  topo_earth_ellipsoid_correction(gp, ht, S, C);
  //printf("topo check %lf (%lf) %lf (%lf)\n", S, 0.546861, C, 0.836339);
  DEG2RAD(lon); DEG2RAD(lat); DEG2RAD(hp); DEG2RAD(lst);
  DEG2RAD(obliq);
  n = cos(lon)*cos(lat) - C * sin(hp) * cos(lst);
  //  printf("lst %lf S %lf C %lf n %lf\n", lst, S, C, n);
  n = 1./n;
  v_lon = n * ( sin(lon) * cos(lat) - sin(hp) * (S * sin(obliq) + C*cos(obliq)*sin(lst)));
  v_lon = atan(v_lon);
  v_lat = n * cos(v_lon) * ( sin(lat) - sin(hp) * (S * cos(obliq) - C*sin(obliq)*sin(lst)));
  v_lat = atan(v_lat);
  RAD2DEG(v_lon);
  RAD2DEG(v_lat);
}
