/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

  main.c file
*/

#include "expression.h"
#include "location_expression.h"
#include "evaluate.h"
#include "parser.h"
#include "lexer.h"
#include <time.h>
#include <stdio.h>
#include <libintl.h>
#include <locale.h>
#include "startime.h"
#include "sun.h"
#include "moon2.h"
#include "moonrise.h"
#include "moon3.h"
#include "planets.h"
#include <math.h>
#include "angle.h"
#include "main.h"
#define MAIN
#include "globals.h"
#include "equationOfTime.h"
#include <getopt.h>
#include "parse_options.h"
#include "self_test.h"
#include "eclipse.h"

#define gettext_noop(String) String

int main(int argc, char *argv[])
{
  SExpression *e = NULL;
  locationExpression *l = NULL;
  char        *errMsg;
  //  sqlite3                  *db;
  int                       success;

  // GNU LOCALE/GETTEXT SUPPORT
  setlocale (LC_ALL, "");
  //  bindtextdomain (PACKAGE, LOCALEDIR);
  //  textdomain (PACKAGE);
  bindtextdomain ("sun", getenv("PWD")); // "/usr/share/locale/");
  textdomain ("sun");
  
  initialize_cvalues();

  /* OPTIONS */
  default_options();
  /* argc argv */
  if (argc > 1){
    //    printf("argv %s %s\n", argv[0], argv[1]);
    parse_options(argc, argv);
  }

  //  printf("\t daylight=%d\n",  daylight);

  /* FINALIZE LOCATION AND OTHER OPTIONS*/
  l = getLocation(sun_options.location);
  if (sun_options.self_verbosity>=3){
    if (l)
      printf("Result of '%s' is (%lf,%lf,%lf)\n", sun_options.location, l->lat, l->lon, l->elevation);
    else
      printf("%s no location specified\n", sun_options.location);
  }
  sun_options.latlon = l;
  if (l){
    // guess at timezone unless timezone specifically supplied (timezone_override)
    sun_options.timezone = guess_timezone(l->lon);
    if (sun_options.self_verbosity>=2) printf("Guessing timezone %lf from location longitude %lf\n", sun_options.timezone, l->lon);
  }
  if (sun_options.self_verbosity>=3) print_options();
  
  /* Set NOW */
  extended dj, dj00, ut=0, lt = 0, tzi, ds, gp, gl, msl_km, alt, az;
  tzi = sun_options.timezone;  // hours west negative 
  ds  = sun_options.daylight;   // hours summer positive
  if (sun_options.latlon==NULL){
    printf(_("Error: location format must specify valid Northing and Easting, e.g. 36N116W\n"));
    printf(_("Orbtimes exiting...\n"));
    exit(0);
  }
  gp = sun_options.latlon->lat;
  gl = sun_options.latlon->lon;
  msl_km = sun_options.latlon->elevation;
  dj = get_now_jdays();
  // set again after guessing DST
  ds  = sun_options.daylight;   // hours summer positive

  /* Set Sunrise / Sunset */
  get_now_sunrise_sunset(dj, tzi, ds, gp, gl, msl_km);

  /* Set TWILIGHT Rise / Set */
  get_now_twilight_rise_set(dj, tzi, ds, gp, gl, msl_km);

  /* Set Moon Rise / Moon Set */
  if (sun_options.self_disp_moon)
    get_now_moonrise_moonset(dj, tzi, ds, gp, gl, msl_km);

  /* Set PLANETS */  
  if (sun_options.self_disp_mercury)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Mercury", MERCURY);
  if (sun_options.self_disp_venus)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Venus", VENUS);
  if (sun_options.self_disp_mars)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Mars", MARS);
  if (sun_options.self_disp_jupiter)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Jupiter", JUPITER);
  if (sun_options.self_disp_saturn)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Saturn", SATURN);
  if (sun_options.self_disp_uranus)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Uranus", URANUS);
  if (sun_options.self_disp_neptune)
    get_now_planetrise_planetset(dj, tzi, ds, gp, gl, msl_km, "Neptune", NEPTUNE);  

  /* Set SUN ALT AZ */
  dj00 = floor(jdays_now)+0.5;    // 0.5 is 00UT
  ut = (jdays_now - dj00)*24.0;// for lt +tzi +ds;
  sun_alt_az(dj00, ut, tzi, ds, gp, gl, msl_km, az, alt);
  sun_azimuth = az;                                         // globals
  sun_altitude = alt;                                       // globals
  if (sun_options.self_verbosity>=1)
    printf(_("Sun's azimuth=%lf degrees clockwise from north and altitude=%lf degrees above horizon\n"), az, alt);

  // PLANETS and MOON NOW POSITION AZ-EL
  double          solar_longitude, solar_radius, d2, d3, am,
    ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, el;  // el alias alt
  sun_position(dj, ut, solar_longitude, solar_radius, d2, d3, am);   

  /* Set Moon AZ-EL (or AlT) */
  if (sun_options.self_disp_moon){
    if (sun_options.self_verbosity>=1){
      extended     phase, distance_to_moon;
      compute_moon_az_el(dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon,
			 ra, dec, value_min, value_max, az, el, phase, distance_to_moon);
      printf(_("Moon azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
      printf(_("\tPhase=%lf\n"), phase*100);
      if (sun_options.self_verbosity>=3) printf(_("\tDistance to moon=%lf km\n"), distance_to_moon);            
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }

  // Planets AZ-EL (or ALT)
  // MERCURY
  if (sun_options.self_disp_mercury){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(MERCURY, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Mercury azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // VENUS
  if (sun_options.self_disp_venus){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(VENUS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Venus azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // MARS
  if (sun_options.self_disp_mars){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(MARS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Mars azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // JUPITER
  if (sun_options.self_disp_jupiter){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(JUPITER, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Jupiter azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // SATURN
  if (sun_options.self_disp_saturn){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(SATURN, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Saturn azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // URANUS
  if (sun_options.self_disp_uranus){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(URANUS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Uranus azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  // NEPTUNE
  if (sun_options.self_disp_neptune){
    if (sun_options.self_verbosity>=1){
      compute_planet_az_el(NEPTUNE, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
      printf(_("Neptune azimuth=%lf degrees clockwise from north and altitude %lf=degrees above horizon\n"), az, el);
    }
    if (sun_options.self_verbosity>=2){
      printf(_("\tRight Ascension (RA)=%lf degrees Declination (DEC)=%lf degrees\n"), ra, dec);
      printf(_("\tEcliptic longitude=%lf latitude=%lf\n"), ecliptic_lon, ecliptic_lat);
    }
  }
  //  if (sun_options.moon_days) 
  if (sun_options.self_disp_moondays) get_moon_days();    

  /* PROCESS QUERY */
  if (sun_options.mode_query) return do_query();

  /* SELF TEST */
  if (sun_options.self_test) do_self_test();
  return 0;
}


double get_now_jdays()
{
  char   timestring[255];
  struct timeval            tv;
  struct timezone           tz;
  /*  time_t  */
  unsigned long long                   // used this for raspberry pi compat
    tv_sec,                            // seconds since 1970
    seconds;                           // seconds since 1900
  extended                  ds,
    now_day, now_month, now_year, now_dayOfWeek;

  /* real-time, self_date_is_now = false, 
     use gettimeofday to get actual wall clock time
   */
  if (sun_options.self_date_is_now == false){
    // 1970 UNIX TIME
    //  UNIX systems represent time in seconds since the Epoch, 1970-01-01 00:00:00 +0000 (UTC)  
    gettimeofday(&tv, &tz);
    seconds = tv.tv_sec;

    // ASCII-ART
    if (sun_options.self_ascii_art == true){
    printf("***************************\n");
    printf(_("    ____   ___  _ _ _\n"
	     "   |  _ \\ / _ \\| | | |\n"
	     "   | | | | |_| | | | |\n"
	     "   |_| |_|\\___/ \\___/\n"
	     "\n"));
    }
    if (sun_options.self_verbosity>=3){
      printf(_("Epoch defined as 1970-01-01 00:00:00 +0000 (UTC)\n"));
      printf(_("Epoch seconds (1970)\t%ld\n"), (unsigned long)seconds);
    }
    // seconds from Jan 1, 1900 until Jan 1, 1970 =  2208988800L
    seconds += 2208988800ULL;
    if (sun_options.self_verbosity>=3){    
      printf(_("Julian seconds (1900)\t%ld\n"), (unsigned long)seconds);
    }
    // JulianDays1900 starts at noon, December 31, 1899. 
    // Thus midnight at greenwich = 0.5
    // days = 24*60*60 seconds
    jdays_now = (extended)seconds / ((extended)24.0*60.0*60.0) + 0.5; // 0.5 is twelve hours from noon to midnight
  }
  /* non-realtime, artificial wall clock, self_date_is_now == true,
     set jdays_now to sun_options.datejd;
  */
  else {
    if (sun_options.self_verbosity>=3) printf("overriding real-time with artificial date (%s)\n", sun_options.date);
    jdays_now = sun_options.datejd + 0.5;
  }
  /* guess at daylight savings */
  if (sun_options.self_guess_daylight_savings == true){
    CalendarDays(jdays_now, now_day, now_month, now_year, now_dayOfWeek);
    ds = guess_daylight_savings(jdays_now, now_year, 1);
    if (sun_options.self_verbosity>=2)    printf(_("Guessing Daylight Savings (hours) DST=%lf \n"), ds);
    sun_options.daylight = ds;
  }

  //  printf("Julian seconds (1900) \t%ld\n", (unsigned long)seconds);
  //  The Julian date for CE  1899 December 31 12:00:00.0 UT is
  //  JD 2415020.000000
  fullTimeString(timestring, jdays_now);
  if (sun_options.self_verbosity>=2){
    printf(_("NOW Universal %s"), timestring);
    printf(_("  Julian days %lf\n"), jdays_now + 2415020.000000);
    printf(_("NOW Julian days (1900) \t%lf\n"), jdays_now);
  }
  // local time  
  fullTimeString(timestring, jdays_now+(sun_options.timezone+sun_options.daylight)/24.0);          
  if (sun_options.self_verbosity>=1){
    printf(_("NOW Local %s\n"), timestring);
  }

  //  printf("***************************\n");
  return jdays_now;
}


void 
get_now_sunrise_sunset(extended dj, 
			      extended tzi, 
			      extended ds,
			      extended gp, 
			      extended gl, 
			      extended msl_km)
{
  extended    dj00, hour;
  int         ihours, iminutes, iseconds;
  extended    au, ad, utu, dju, utd, djd;
  int         never_rise, never_sets;
  char        timestring[255];

#if 0
  dj = jdays_now; //floor(jdays_now)+0.5;    // 0.5 is 00UT
  tzi = -8;                   // nevada  west negative
  ds = 0;                     // daylight savings
  gp = 36.048152;
  gl = -114.951;
  gp = 36.1666;
  gl = -115.1666;
  msl_km = 0;
#endif

  if (sun_options.self_verbosity>=3)
    printf("Debug get_now_sunrise_sunset DJ=%lf TZ=%lf , DS=%lf, LAT=%lf LON=%lf ",
	   dj, tzi, ds, gp, gl);
  //   printf("debug sunrise/sunset utu %lf utd %lf\n", (utu+tzi+ds), (utd+tzi+ds));

  hour = (jdays_now - floor(dj)-0.5)*24.0;
  dj00 = floor(jdays_now)+0.5;    // 0.5 is 00UT
  sun_calculate_rise_tilnow(utu, dju, utd, djd, dj00, tzi, ds, gl, gp, jdays_now, hour, msl_km, never_rise, never_sets, au, ad );

  jdays_sunset = djd + utd/24.0;
  jdays_sunrise = dju + utu/24.0;

  //  printf("Julian days (1900) %lf\n", jdays_now);
  //  printf("Julian days %lf\n", jdays_now+2415020.000000);
  //  printf("*************************************\n");

  if (jdays_sunset > jdays_sunrise){
    if (sun_options.self_verbosity>=1) printf(_("Sunset > Sunrise. (Night)\n"));
    nighttime = true;
    daytime   = false;

    // ASCII-ART
    if (sun_options.self_ascii_art == true){
    printf(_("          (_)     | |     _                  \n"
"     ____  _  ____| |__ _| |_                 \n"
"    |  _ \\| |/ _  |  _ (_   _)                \n"
"    | | | | ( (_| | | | || |_                 \n"
"    |_| |_|_|\\___ |_| |_| \\__)       \n"
"            (_____|                           \n"
	     "\n"));
    }

    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("Sunrise %s (local day)"), timestring);
    if (sun_options.self_verbosity>=2)
      printf(_("  Julian %lf (hours UT=%lf )\n"),  utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("Sunset  %s (local day)"), timestring);
    if (sun_options.self_verbosity>=2)
      printf(_("  Julian %lf (hours UT=%lf ) \n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
  }
  else
  if (jdays_sunset <= jdays_sunrise){
    if (sun_options.self_verbosity>=1) printf(_("Sunrise > Sunset. (Day)\n"));
    nighttime = false;
    daytime   = true;
    // ASCII-ART
    if (sun_options.self_ascii_art == true){
    printf(_("     _\n"
"    | |\n"
"  __| |_____ _   _\n"
" / _  (____ | | | |\n"
"( (_| / ___ | |_| |\n"
" \\____\\_____|\\__  |\n"
	     "            (____/\n"));
    }

    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("Sunset local day %s"), timestring);
    if (sun_options.self_verbosity>=2)
      printf(_(" Julian %lf (hours UT=%lf)\n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("Sunrise local day %s"), timestring);
    if (sun_options.self_verbosity>=2)
      printf(_(" Julian %lf (hours UT=%lf)\n"), utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
  }
			 
  //  printf("*************************************\n");
  //  return jdays_sunset;
}

void
get_now_twilight_rise_set(double dj, 
			      double tzi, 
			      double ds,
			      double gp, 
			      double gl, 
			      double msl_km)
{
  extended    dj00, hour;
  int         ihours, iminutes, iseconds;
  extended    au, ad, utu, dju, utd, djd, rad_twilight;
  int         never_rise, never_sets;
  char        timestring[255];


  hour = (jdays_now - floor(dj)-0.5)*24.0;
  dj00 = floor(jdays_now)+0.5;    // 0.5 is 00UT
  switch (sun_options.mode_twilight){
  case CIVIL_TWILIGHT:
    rad_twilight = 1.047198e-1;      // civil
    break;
  case NAUTICAL_TWILIGHT:
    rad_twilight = 2.094395e-1;  //nautical
    break;
  case ASTRONOMICAL_TWILIGHT:
    rad_twilight = 3.141593e-1;  //astronomical
    break;
  case GENERAL_TWILIGHT:
    rad_twilight = sun_options.twilight_angle;
    break;
  default:
    rad_twilight = 1.047198e-1;      // civil
    break;
  }
  sun_calculate_twilight_tilnow(utu, dju, utd, djd, dj00, tzi, ds, gl, gp, jdays_now, hour, msl_km, never_rise, never_sets, au, ad, rad_twilight);
  // REGISTER GLOBALS JD..Rise JD..Set
  jdays_twilight_set = djd + utd/24.0;
  jdays_twilight_rise = dju + utu/24.0;

  // FROM HERE VERBOSE PRINT
  //  if (sun_options.self_verbose == false) return;

  // NOW 
  //  printf("Julian days (1900) %lf\n", jdays_now);
  //  printf("Julian days %lf\n", jdays_now+2415020.000000);
  if (sun_options.self_disp_twilight==true){
    // TWILIGHT Rise
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("TWILIGHT Rise %s (local day)"), timestring);
    if (sun_options.self_verbosity>=2)    printf(_("\tJulian %lf (hours UT=%lf)\n"), utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
    // TWILIGHT Set
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("TWILIGHT Set  %s (local day)"), timestring);
    if (sun_options.self_verbosity>=2)    printf(_("\tJulian %lf (hours UT=%lf)\n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
  }
  // twilight_rise > sunrise --> in twilight rise
  // twilight_set > sunset --> in twilight set
  ///day  if (jdays_sunset > jdays_sunrise){ 
  //nigth    if (jdays_sunset > jdays_sunrise){
  // end of day 
  /* if ((daytime && jdays_twilight_set > jdays_sunrise) ||  */
  /*     (nighttime && jdays_twilight_rise > jdays_sunset)){ */
  /*   printf("                 _ _  _       _         \n"); */
  /*   printf("       _        (_) |(_)     | |     _  \n"); */
  /*   printf("     _| |_ _ _ _ _| | _  ____| |__ _| |_  \n"); */
  /*   printf("    (_   _) | | | | || |/ _  |  _ (_   _) \n"); */
  /*   printf("      | |_| | | | | || ( (_| | | | || |_ \n"); */
  /*   printf("      \\__) \\___/|_|\\_)_|\\____|_| |_|\\__)  \n"); */
  /*   printf("			(____|            \n"); */
  /*   printf("\n"); */
  /* } */
}

void 
get_now_moonrise_moonset(extended dj, 
			      extended tzi, 
			      extended ds,
			      extended gp, 
			      extended gl, 
			      extended msl_km)
{
  extended    dj00, hour;
  int         ihours, iminutes, iseconds, rise_converges = true, set_converges = true;
  extended    au, ad, utu, dju, utd, djd;
  int         never_rise, never_sets;
  char        timestring[255];

  hour = (jdays_now - floor(dj)-0.5)*24.0;
  dj00 = floor(jdays_now)+0.5;    // 0.5 is 00UT
  moon_calculate_rise_tilnow(utu, dju, utd, djd, dj00, tzi, ds, gl, gp, jdays_now, hour, never_rise, never_sets, au, ad,
			     rise_converges, set_converges);
  jdays_moonset = djd + utd/24.0;
  jdays_moonrise = dju + utu/24.0;
  jdays_moon_azimuth_set = ad;
  jdays_moon_azimuth_rise = au;

  //  printf("Julian days (1900) %lf\n", jdays_now);
  //  printf("Julian days %lf\n", jdays_now+2415020.000000);
  //  printf("*************************************\n");

  if (jdays_moonset > jdays_moonrise){
    moon_nighttime = true;
    moon_daytime   = false;
    if (sun_options.self_verbosity>=1) printf(_("Moon Set > Moon Rise. (Moon Night)\n"));
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("Moon Rise local day %s"), timestring);
    if (sun_options.self_verbosity>=2) printf(_("  Julian %lf (hours UT=%lf )\n"),  utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("Moon Set local day %s"), timestring);
    if (sun_options.self_verbosity>=2) printf(_("  Julian %lf (hours UT=%lf ) \n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
  }
  else
  if (jdays_moonset <= jdays_moonrise){
    if (sun_options.self_verbosity>=1) printf(_("Moon Rise > Moon Set. (Moon Day)\n"));
    moon_nighttime = false;
    moon_daytime   = true;
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("Moon Set local day %s"), timestring);
    if (sun_options.self_verbosity>=2) printf(_(" Julian %lf (hours UT=%lf)\n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("Moon Rise local day %s"), timestring);
    if (sun_options.self_verbosity>=2) printf(_(" Julian %lf (hours UT=%lf)\n"), utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
  }
  if (sun_options.self_verbosity>=2){
    printf(_("Moon  Azimuth Rise %lf Set %lf degrees clockwise from north\n"), jdays_moon_azimuth_rise, jdays_moon_azimuth_set);
  }
  //  printf("*************************************\n");
}

void 
get_now_planetrise_planetset(double dj, 
			     double tzi, 
			     double ds,
			     double gp, 
			     double gl, 
			     double msl_km,
			     char *planet_name,
			     short planet_index)
{
  double    dj00, hour, au, ad, utu, dju, utd, djd;
  int         
    ihours, iminutes, iseconds, 
    rise_converges = true, set_converges = true,
    never_rise, never_sets;
  char        timestring[255];
  double      
    *jdays_aplanet_set=&jdays_planets_set[planet_index], 
    *jdays_aplanet_rise=&jdays_planets_rise[planet_index],
    *jdays_aplanet_azimuth_set=&jdays_planets_azimuth_set[planet_index], 
    *jdays_aplanet_azimuth_rise=&jdays_planets_azimuth_rise[planet_index];    
  int         *aplanet_nighttime=&planets_nighttime[planet_index], *aplanet_daytime=&planets_daytime[planet_index];

  hour = (jdays_now - floor(dj)-0.5)*24.0;
  dj00 = floor(jdays_now)+0.5;    // 0.5 is 00UT
  //  planet_calculate_rise_tilnow(planet_index, utu, dju, utd, djd, dj00, tzi, ds, gl, gp, jdays_now, hour, never_rise, never_sets, au, ad,
  //			       rise_converges, set_converges);
  planet_calculate_rise(planet_index, utu, utd, dj00, tzi, ds, gl, gp, never_rise, never_sets, au, ad);
  // planets are far away, rise and set don't change much from day to day.
  if (utu > hour) dju = dj00-1;  // today's rise > now, so use yesterday's rise
  else dju = dj00;
  if (utd > hour) djd = dj00-1;  // today's set > now, so use yesterday's set
  else djd = dj00;

  // for queries
  *jdays_aplanet_set = djd + utd/24.0;
  *jdays_aplanet_rise = dju + utu/24.0;
  *jdays_aplanet_azimuth_rise = au;     // pilot degrees
  *jdays_aplanet_azimuth_set = ad;      // pilot degrees
  
  if (*jdays_aplanet_set > *jdays_aplanet_rise){
    *aplanet_nighttime = true;
    *aplanet_daytime   = false;
    if (sun_options.self_verbosity>=2) printf(_("%s Set > %s Rise. (%s Night)\n"), planet_name, planet_name, planet_name);
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("%s Rise local day %s"), planet_name, timestring);
    if (sun_options.self_verbosity>=2) printf(_("  Julian %lf (hours UT=%lf )\n"),  utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("%s Set local day %s"), planet_name, timestring);
    if (sun_options.self_verbosity>=2) printf(_("  Julian %lf (hours UT=%lf ) \n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
  }
  else
  if (*jdays_aplanet_set <= *jdays_aplanet_rise){
    if (sun_options.self_verbosity>=2) printf(_("%s Rise > %s Set. (%s Day)\n"), planet_name, planet_name, planet_name);
    *aplanet_nighttime = false;
    *aplanet_daytime   = true;
    fullTimeString(timestring, djd+(utd+tzi+ds)/24.);
    printf(_("%s Set local day %s"), planet_name, timestring);
    if (sun_options.self_verbosity>=2) printf(_(" Julian %lf (hours UT=%lf)\n"),  utd/24.+djd + 2415020.000000, utd);
    else printf("\n");
    fullTimeString(timestring, dju+(utu+tzi+ds)/24.);
    printf(_("%s Rise local day %s"), planet_name, timestring);
    if (sun_options.self_verbosity>=2) printf(_(" Julian %lf (hours UT=%lf)\n"), utu/24.+dju + 2415020.000000, utu);
    else printf("\n");
  }
  if (sun_options.self_verbosity>=2){
    printf(_("%s Azimuth Rise %lf Set %lf degrees clockwise from north\n"), planet_name, *jdays_aplanet_azimuth_rise, *jdays_aplanet_azimuth_set);
  }
}

/* MOONDAYS. lunar count, New and Full oon.
 * ref. Peter Duffett-Smith 6800 MOONNNF
 */
void
get_moon_days()
{
  int i,j;
  const int isnew[2] = {0, 1};
  extended d,m,y,dow, argument_latitude[4], event_day[4], event_fraction[4];
  extern char 						*threeLetterMonthStringArray[];
  const char *fullStr=gettext_noop("\tFull\t%2d %s %d\n");
  const char *newStr=gettext_noop("\tNew\t%2d %s %d\n");

  CalendarDays(jdays_now, d, m, y, dow);
  /* for (j=0;j<2;j++){ */
  /*   for (i=0;i<2;i++){ */
  /*     newmoon(1, m-j, y, isnew[i], argument_latitude[i+j], event_day[i+j], event_fraction[i+j]); */
  /*     //      newmoon(d, m-j, y, isnew[i], argument_latitude[i+j], event_day[i+j], event_fraction[i+j]); */
  /*   } */
  /* } */
  newmoon(1, m, y, 0, argument_latitude[0], event_day[0], event_fraction[0]);
  newmoon(1, m, y, 1, argument_latitude[1], event_day[1], event_fraction[1]);
  newmoon(31, m, y, 0, argument_latitude[2], event_day[2], event_fraction[2]);
  newmoon(31, m, y, 1, argument_latitude[3], event_day[3], event_fraction[3]);

  printf(_("New and Full Moons\n"));
  //  if (jdays_now<event_day[0]) printf(_("\tToday\n"));
  for (i=0;i<4;i++){
    CalendarDays(event_day[i]+event_fraction[i], d, m, y, dow);
    if (i>0&&jdays_now>event_day[i-1]&&jdays_now<event_day[i]) 
      printf(_("\tToday, %d days from, %d days until\n"), (int)(jdays_now-event_day[i-1]), (int)(event_day[i]-jdays_now+1));
    if (i%2==0) printf(newStr, (int)d, threeLetterMonthStringArray[(int)m-1], (int)y);
    else printf(fullStr, (int)d, threeLetterMonthStringArray[(int)m-1], (int)y);
  }
}

bool do_query()
{
  return logic_withcheck(sun_options.query);
}

