/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
/* After so many hardware platforms, and operating systems, I have found it
   necessary to doubt results of all numerical floating point calculations.
   Hence the necessity to benchmark all results with prior values from 
   Peter Duffett-Smith's book page 170: 6600 MOONRS
*/

//#include "macmac.h"
//#include "menu.h"
//#include <Picker.h>	
//#include <Scrap.h>
//#include <Resources.h>
//#include <SegLoad.h>
#include <math.h>
#include <stdio.h>
#include <strings.h>
#include "startime.h"
//#include "application.h"

//#include <OSUtils.h>
//#include <TextUtils.h>
//#include "util.h"
#include "angle.h"
//#include <SANE.h>
//#include "alerts.h"
//#include "preferences.h"
#include "sun.h"
#include "moon2.h"
#include "moonrise.h"


#define DEG2RAD(x) 1.745329252e-2L * (x)
#define RAD2DEG(x) 5.729577951e1L * (x)
#define SQR(x) (x)*(x)
const extended TP = 2 * M_PI;
/*
 * gl = geographic lon (degrees),  
 * gp = geographic lat (degrees)
 */
void moon_calculate_rise(extended &utu, extended &utd, 
			 extended dj, extended tz, extended ds,
			 extended gl, extended gp, int& never_rises, int& never_sets,
			 extended &au, extended &ad, int &rise_converges, int &set_converges)
{
  const int      iter_limit = 30;
  int            iter = 0, icase;
  extended 	 djp, di = 1.454441e-2, sr, rr, d2, d3,
    dp, x, y, p, q, d0, lu,
    utm, ut, ld, mm, bm, pm, distance_to_moon, phase, 
    solar_longitude, solar_radius, a;

  rise_converges = true;
  set_converges = true;
  for (icase=0; icase<2; icase++){
    iter = 0;
    ut  =  0.0;
    utm = 12.0 -( tz + ds);  // universal for local midday
    
    while (fabs(utm - ut) > 0.01 && iter++ < iter_limit){
      ut = utm;              // universal times
      a = ut + tz + ds;      // local time
      /*      print_hours("a", a); */
      if (a > 24.0) calculate_moon((djp = dj-1), ut, mm, bm, pm, distance_to_moon, phase, 
				   solar_longitude, solar_radius,true);
      else if (a < 0.0) calculate_moon((djp = dj+1), ut, mm, bm, pm, distance_to_moon, phase, 
				       solar_longitude, solar_radius,true);
      else calculate_moon((djp = dj), ut, mm, bm, pm, distance_to_moon, phase, 
			  solar_longitude, solar_radius,true);
      // sun_position(dj, utm, sr, rr, d2, d3);
      // mm = moons ecliptic longitude
      // bm = moons ecliptic latitude
      // pm = moons horizontal parallax
      // dj is the day number.
      //nutation(dj, dp, d0);
      x =  mm;// + dp;                                              // units are degrees.longitude
      y =  bm;                                                      // latitude
      di = 2.7249e-1 * sin(DEG2RAD(pm)) + 9.8902e-3 - DEG2RAD(pm);  // parallax
      geocentric_ecliptic_latlon_to_radec(-1, p, q, x, y, dj );
      // p,q units are degrees
      time_of_rising(p, q, di, gp, lu, ld, ad, au, never_sets, never_rises);
      if (icase == 0) 
	time_sidereal_to_universal(utm, ds, tz, gl, dj, lu);
      else
	time_sidereal_to_universal(utm, ds, tz, gl, dj, ld);      
    }
    //  utm = floor(djp) + utm / 24.0;               // convert utm to julian time.
    if (never_rises){
      utm = -1.0;
      printf("never rises\n");
    }
    if (never_sets){
      utm = -1.0;
      printf("never sets\n");
    }
    if (icase == 0){
      utu = utm;
      if (iter>=iter_limit) rise_converges = false;
    }
    else {
      utd = utm;
      if (iter>=iter_limit) set_converges = false;
    }
    /* printf("djp %lf\n", djp);*/
  }
}

void moon_calculate_rise_tilnow(extended &utu, extended &dju, extended &utd, extended &djd,
				extended dj, extended tz, extended ds,
				extended gl, extended gp, extended now, int now_hour, int& never_rises, int& never_sets,
				extended &au, extended &ad, int &rise_convergent, int &set_convergent )
{
  const int      iter_limit = 30;
  int            iter = 0, icase, rise_converges, set_converges;
  extended 	 djp, di = 1.454441e-2, sr, rr, d2, d3,
    dp, x, y, p, q, d0, lu,
    utv, utm, ut, ld, mm, bm, pm, distance_to_moon, phase, 
    solar_longitude, solar_radius, a;

  rise_converges = true;
  set_converges = true;

  //  printf("now hour %lf ut %lf\n", now_hour, ut);
  // icase=0 sunrise utu  lu
  // icase=1 sunset  utd  ld
  for (icase=0;icase<2;icase++){
    iter = 0;
    ut  = 100.0;    // large number
    utv = now_hour; // 1st guess twelve hours earlier from now UT
    djp = dj;
    while (fabs(utv - ut) > 0.01 && iter++ < 10){
      //      printf("icase=%d iter %d ut %lf utv  %lf\n", icase, iter, ut, utv);
      ut = utv;          // save last guess for event

      //      a = ut + tz + ds;  // event local time
      // event must be before now
      // (djp,a) < (dj,now)
      //
      // the fine point here is that sun's position is calc'd 
      // first from it's Julian date. In considering local time,
      // the date might be pushed onto another day. Effect
      // is non-trivial.

      //      printf("til now(%d) %lf %lf %lf %lf \n", icase, dj, ut, dj+ut/24.0, now);
      if (djp+ut/24.0 > now)	 djp = djp-1;  // next local day
      if (djp+ut/24.0 > now)	 djp = djp-1;  // next local day      
      //      else if (a < 0.0)	 sun_position((djp = dj-1), utv, sr, rr, d2, d3, am);  // previous local day
      //      sun_position(djp, utv, sr, rr, d2, d3, am);    // same day
      calculate_moon(djp, utv, mm, bm, pm, distance_to_moon, phase, solar_longitude, solar_radius,true);      
 
      //      nutation(djp, dp, d0);
      x =  mm;// + dp;                                              // units are degrees.longitude
      y =  bm;                                                      // latitude
      di = 2.7249e-1 * sin(DEG2RAD(pm)) + 9.8902e-3 - DEG2RAD(pm);  // parallax
      geocentric_ecliptic_latlon_to_radec(-1, p, q, x, y, djp ); // p,q units are degrees
      // compute sunup/sundown sidereal time lu/ld 
      time_of_rising(p, q, di, gp, lu, ld, ad, au, never_sets, never_rises);
      if (never_sets == true || never_rises == true) return;                // no change
      if (icase == 0) time_sidereal_to_universal(utv, ds, tz, gl, djp, lu);  // sunup
      else time_sidereal_to_universal(utv, ds, tz, gl, djp, ld);             // sundown
    }
    if (icase == 0){ utu = utv; dju = djp; }
    else { utd = utv; djd = djp; }
  }
}
