/*  Copyright (C) 2017 Nicholas C. Strauss (strauss@positive-internet.com)
    This file is part of ORBTIMES.

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "expression.h"
#include "location_expression.h"
#include "evaluate.h"
#include "parser.h"
#include "lexer.h"
#include <time.h>
#include <stdio.h>
#include "startime.h"
#include "sun.h"
#include <math.h>
#include "angle.h"
#include "main.h"
#define MAIN extern
#include "globals.h"
#include "equationOfTime.h"
#include <getopt.h>
#include "parse_options.h"
#include "self_test.h"
#include "moon2.h"
#include "moonrise.h"
#include "planets.h"
#include "moon3.h"
#include "eclipse.h"

#define POSITIVE_HOURS(x) ((x)<0?(x)+24.0:(x))

static int saros_check();

SExpression *getAST(const char *expr)
{
  SExpression            *expression;
  locationExpression     *lexpression;
  yyscan_t                scanner;
  YY_BUFFER_STATE         state;
  int                     success;

  if (yylex_init(&scanner)) {
    // couldn't initialize
    return NULL;
  }

  state = yy_scan_string(expr, scanner);

  success = yyparse(&expression, &lexpression, scanner);
  if (success) {
    // error parsing
    printf("parse error on %s\n", expr);
    return NULL;
  }

  yy_delete_buffer(state, scanner);

  yylex_destroy(scanner);

  return expression;
}

bool
logic_withcheck(char const *test)
{
  SExpression *e = getAST(test);
  double result = evaluate(e);
  deleteExpression(e);
  if (isnan(result)){
    printf("Result of '%s' is FALSE\n", test);// result, result+2415020.000000);
    return false;
  }
  printf("Result of '%s' is TRUE\n", test); // result, result+2415020.000000);  
  return true;
}

locationExpression *getLocation(const char *expr)
{
  SExpression *expression;
  locationExpression *lexpression;
  yyscan_t scanner;
  YY_BUFFER_STATE state;
  int             success;

  if (yylex_init(&scanner)) {
    // couldn't initialize
    return NULL;
  }

  state = yy_scan_string(expr, scanner);

  success = yyparse(&expression, &lexpression, scanner);
  if (success) {
    // error parsing
    printf("getLocation parse error %s error code=%d\n", expr, success);
    return NULL;
  }

  yy_delete_buffer(state, scanner);

  yylex_destroy(scanner);

  return lexpression;
}

void do_self_test()
{
  bool        check = true;
  double result = 0;
  //  char test[]="sunrise<now<sunset";
  char test[]="3.1<3.116<3.12";
  char test2[]="114W33.1245 43N32 2.4MSL";
  char test_location[]="114~33.1245W 43~32N 2400ft";
  char test_location2[]="114.5W 43.6N 2400ft";
  char test_location3[]="114~33'33.1245W 43~32'22.11N 2400ft";
  SExpression *e = NULL;
  locationExpression *l = NULL;
  struct timeval            tv;
  struct timezone           tz;
  time_t                    
    tv_sec,                            // seconds since 1970
    seconds;                           // seconds since 1900
  l = getLocation(test_location);
  printf("LOCATION '%s' is latitude=%lf longitude=%lf elevation=%lf\n", test_location, l->lat, l->lon, l->elevation);

  l = getLocation(test_location2);
  printf("LOCATION2 '%s' is latitude=%lf longitude=%lf elevation=%lf\n", test_location2, l->lat, l->lon, l->elevation);

  l = getLocation(test_location3);
  printf("LOCATION2 '%s' is latitude=%lf longitude=%lf elevation=%lf\n", test_location3, l->lat, l->lon, l->elevation);

  e = getAST(test);

  result = evaluate(e);

  printf("Result of '%s' is %f\n", test, result);

  deleteExpression(e);

  check &= logic_withcheck("3.1<3.115<3.12");
  check &= logic_withcheck("+100.0>99.0>60.0>-100.0");
  check &= logic_withcheck("4<5<6<7<8");
  check &= logic_withcheck("4<5>3");
  check &= logic_withcheck("4<5&3>2");
  check &= logic_withcheck("3.1<3.115<3.12");  
  check &= logic_withcheck("sunrise<sunrise+(1/24)");
  check &= logic_withcheck("sunset<sunset+(1/24)");
  check &= logic_withcheck("sunset<now");
  check &= logic_withcheck("sunrise<now");      
  check &= logic_withcheck("1*24");  
  check &= logic_withcheck("1/24");  
  check &= logic_withcheck("jan<feb<mar<apr<may<jun<jul<aug<sep<oct<nov<dec");
  if (check) printf("LOGIC CHECKS PASSED\n");
  else  printf("LOGIC CHECKS FAILED\n");
  // true astonomical reckoned julian days 1 Jan 4713 BC (1/1/-4712)
  extended check_day, check_month, check_year, check_dayOfWeek;
  CalendarDays(-2415020, check_day, check_month, check_year, check_dayOfWeek);
  printf("check: day %lf month %lf year %lf dow %lf\n",
	 check_day, check_month, check_year, check_dayOfWeek);	 
  
  // 1970 UNIX TIME
  //  UNIX systems represent time in seconds since the Epoch, 1970-01-01 00:00:00 +0000 (UTC)  
  gettimeofday(&tv, &tz);
  printf("gettimeofday timezone info minutes west of Greenwich %d dst %d\n",
	 tz.tz_minuteswest,
	 tz.tz_dsttime);
  seconds = tv.tv_sec;
  printf("Epoch defined as 1970-01-01 00:00:00 +0000 (UTC)\n");
  printf("\tEpoch seconds (1970)\t%ld\n", seconds);
  // seconds from Jan 1, 1900 until Jan 1, 1970 =  2208988800L
  seconds += 2208988800UL;

  // JulianDays1900 starts at noon, December 31, 1899. 
  // Thus midnight at greenwich = 0.5
  // days = 24*60*60 seconds
  extended jdays_now = (extended)seconds / ((extended)24.0*60.0*60.0) + 0.5;
  printf("\tJulian 1900 seconds (1900) \t%ld\n", seconds);
  printf("\tJulian 1900 days \t%lf\n", jdays_now);


  now_hour = (jdays_now - 0.5 - floor(jdays_now-0.5)) * 24.0;
  printf("check hour %lf\n", now_hour);

  CalendarDays(jdays_now, now_day, now_month, now_year, now_dayOfWeek);
  printf("TODAY %d %d %d %d DOW %d\n",
	 (int)jdays_now, (int)now_day, (int)now_month, (int)now_year, (int)now_dayOfWeek);

  extended    dj, tx, ds, gp, gl, msl_km;
  int         ihours, iminutes, iseconds;
  //  char        buffer[80];
  extended    au, ad, utu, dju, utd, djd, tzi, ltu, ltd, vtu, vtd;
  int         never_rise, never_sets;


  /*  dj = JulianDays(21, 9, 1997); */
  dj = jdays_now;
  tzi = -8;                   // nevada  west negative
  ds = 1;                     // daylight savings
  gp = 36.048152;
  gl = -114.951;
  msl_km = 0;
  //  gp = hms2X(39, 37, 0);
  //  gl = hms2X(-75, 36, 0);
  sun_calculate_rise(utu, dju, utd, djd, dj, tzi, ds, gl, gp, msl_km, never_rise, never_sets, au, ad );
  compute_hours(utu+tzi+ds, &ihours, &iminutes, &iseconds);
  printf("TODAY SUNRISE %d (%lf) %d:%d:%d\n", (int)dju, utu+tzi+ds, ihours, iminutes, iseconds);

  compute_hours(utd+tzi+ds, &ihours, &iminutes, &iseconds);
  printf("TODAY SUNSET %d (%lf) %d:%d:%d\n", (int)dj, utd+tzi+ds, ihours, iminutes, iseconds);

  double             local_noon = 12  + tzi + ds - gl/15;
  //  dj = JulianDays(19, 7, 2016);
  //  time_sidereal_to_universal(local_noon, ds, tzi, gl, dj+1, 12.0);
  //  local_noon += tzi + ds - gl/15;
  /* noon calculation based on 
   * College Mathematics Journal Vol 34 No 4, September 2003, 
   * Teets, Predicting Sunrise and Sunset Times
   */
  double minutes;
    //    doy = dayOfYear(dj), // JulianDays(20, 7, 2016) - JulianDays(1, 1, 2016),
  //    minutes = 720 - 10*sin(4*M_PI / 365.25 *(doy - 80)) + 8*sin(2*M_PI/365.25*doy);
  //  minutes += (tzi+ds-gl/15)*60;
  //  ihours = floor(minutes/60);
  //  iminutes = floor((minutes/60-ihours)*60);
  //  compute_hours(local_noon, &ihours, &iminutes, &iseconds);
  sun_local_noon_transit(dj, tzi, ds, gl, minutes, ihours, iminutes);
  printf("TODAY NOON (MERIDIAN TRANSIT) %d:%d\n", ihours, iminutes);
  printf("DURATION AM %lf PM %lf minutes\n", minutes - 60*positive_hours(utu+tzi+ds),  60*positive_hours(utd+tzi+ds) - minutes);
  
  extended T, eot, lambda, dec, UTrise, UTset;
#if 0
  // analemma calculation
  //  (%i14) analemma: read_matrix ("analemma1.txt");
  // plot2d([discrete,transpose(analemma)[2],transpose(analemma)[3]]);
  //  plot2d([discrete,transpose(analemma)[3],transpose(analemma)[2]],[ylabel,"equation of time"],[xlabel,"declination"]);
  dj = JulianDays(1, 1, 2016);
  printf("day T DEC\n");
  for (int i = 0; i<365; i++){
    T = centuries_j2000(dj+i);  
    eot = equation_of_time(T, 12-tzi-ds, lambda, dec); // local noon
    printf("%d,%lf,%lf\n", i, eot, dec);
  }
  // months
  for (int i=1; i<=12; i++){
    char  *smonth[]={"jan", "feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec"};
    dj = JulianDays(1, i, 2016);
    T = centuries_j2000(dj);  
    eot = equation_of_time(T, 12-tzi-ds, lambda, dec); // local noon
    printf("[\"%s\",%lf,%lf],\n", smonth[i-1], dec, eot);
  }
#endif
  T = centuries_j2000(dj);  
  eot = equation_of_time(T, 12-tzi-ds, lambda, dec); // local noon
  minutes = 720 - eot;
  minutes += (tzi+ds-gl/15)*60;
  ihours = floor(minutes/60);
  iminutes = floor((minutes/60-ihours)*60);
  printf("EOT %lf %lf %lf %d:%d\n", eot, minutes, minutes/60, ihours, iminutes);
  printf("DURATION AM %lf PM %lf minutes\n", minutes - 60*positive_hours(utu+tzi+ds),  60*positive_hours(utd+tzi+ds) - minutes);
  UTrise = calculate_rise_time(T, gp, gl, +1, msl_km*1000);
  UTset = calculate_rise_time(T, gp, gl, -1, msl_km*1000);
  printf("UT rise = %lf set = %lf\n", UTrise, UTset); //  msl_km*1000));
  compute_hours(UTrise+tzi+ds, &ihours, &iminutes, &iseconds);
  printf("USNO rise %d:%d:%d\n", ihours, iminutes, iseconds);
  compute_hours(UTset+tzi+ds, &ihours, &iminutes, &iseconds);
  printf("USNO set %d:%d:%d\n", ihours, iminutes, iseconds);  

  
  //  check = selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 1, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 2);
  //  selftest_ephemeris(dj, tzi, ds, gp, gl, 0, 15);

  /* spot checking. June 26, 2016. Fairbanks AK, Miami FL, 
   */
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 1, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+49/60., 18+16/60., 10.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 2, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+40/60., 18+32/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 3, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+18/60., 18+42/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 4, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 5+52/60., 18+51/60., 10.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 5, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 5+35/60., 19+02/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 6, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 5+32/60., 19+14/60., 10.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 7, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 5+42/60., 19+14/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 8, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 5+53/60., 18+58/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 9, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+0/60., 18+32/60., 10.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 10, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+8/60., 18+6/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 11, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+22/60., 17+52/60.);
  check &= selftest_ephemeris_withcheck("mauna kea 4.2km", 17, 12, 2016, -10, 0, 19+50.0/60.0, -(155+28.0/60.0), 4.2, 6+40/60., 17+57/60.);  
  check &= selftest_ephemeris_withcheck("Fairbanks AK", 26, 6, 2016, -8, 0, (64+49.0/60.0), -(147+45.0/60.0), 0, 3+4/60., 0+43/60.);
  check &= selftest_ephemeris_withcheck("Miami FL", 26, 6, 2016, -5, 1, (25+47.0/60.0), -(80+13.0/60.0), 0, 6+32/60., 20+16/60.);
  check &= selftest_ephemeris_withcheck("Honolulu HI", 26, 6, 2016, -10, 0, (21+19.0/60.0), -(157+50.0/60.0), 0, 5+52/60., 19+17/60.);
  check &= selftest_ephemeris_withcheck("Seattle WA", 26, 6, 2016, -8, 1, (47+38.0/60.0), -(122+20.0/60.0), 0, 5+13/60., 21+11/60.);
  check &= selftest_ephemeris_withcheck("Cambridge MA", 26, 6, 2016, -5, 1, (42+22.0/60.0), -(71+7.0/60.0), 0, 5+9/60., 20+25/60.);
  check &= selftest_ephemeris_withcheck("London UK", 26, 6, 2016, 0, 0,  51.514125, -.093689, 0, 3+45/60., 20+22/60.);
  check &= selftest_ephemeris_withcheck("Tel Aviv IS", 26, 6, 2016, +2, 0, 32.084139, 34.799946, 0, 4+36/60., 18+51/60.);
  check &= selftest_ephemeris_withcheck("Tokyo JP", 26, 6, 2016, +9, 0, 35.685, 139.751389, 0, 4+27/60., 19+1/60.);        
  // Munich, Germany "48.22N 11.54E 0ft"  jmd@gnu.org. reference values U.S. Naval Observatory, Astronomical Applications Department (48N13,11E32
  check &= selftest_ephemeris_withcheck("Munich Germany 0km", 23, 7, 2017, +2, 0, 48.22, 11.54, 0, 5+38/60., 21+2/60.); 
  check &= selftest_anyrise();
  check &= selftest_sunrise();
  check &= selftest_sunposition();

  printf("check %d\n", check);


  /* equation of time test dec,E,lambda
  (%i76) selftest:equation_of_time_full(centuries_j2000(julian(23,8,1984)), 19+5);
  (%o76)    [11.11485940000455, - 2.401115405012291, 151.0140464918149]
  */
  printf("EQUATION OF TIME\n");
  dj = JulianDays(23, 8, 1984);
  T = centuries_j2000(dj);
  eot = equation_of_time(T, 19.+5., lambda, dec);
  if (lambda<0.0) lambda+=360;
  if (dec<0.0) dec+=360;
  printf("dj %lf T %lf eot %lf dec %lf lambda %lf\n",
	 dj, T, eot, dec, lambda);
  printf("\tcheck [dec=11.11485940000455 degrees, eot=-2.401115405012291 minutes, lambda=151.0140464918149 degrees]\n");
  if (fabs(eot+2.401115405012291)<0.1 && 
      fabs(lambda-151.0140464918149)<0.1 && 
      fabs(dec-11.11485940000455)<0.1) check &= 1;     // true is 1
  else check &= 0;                                     // false is 0

  // sidereal time tests
  double  ut; 
  dj = JulianDays(11, 3, 1990);
  ut = 17.+21./60.+43.70/3600.;
  gl = -(148.+31./60.+52.33/3600.);
  //  gp = 38+55/60;
  double mst = local_mean_sidereal_time(dj, ut, gl);
  double gmst = greenwich_mean_sidereal_time(dj, ut), v_mst = 18+44/60.+1.73/3600., v_gmst=4+38./60.+9.22/3600.;
  printf("sidereal time check mst %lf (%lf) gmst %lf (%lf)\n", mst, v_mst, gmst, v_gmst);
   if (fabs(v_mst-mst)<1e-4&&fabs(v_gmst-gmst)<1e-4) check&=1;
  else check &=0;
  printf("check %d\n", check);

  double lst = local_sidereal_time(dj, ut, 1990, gl);
  printf("lst %lf mst %lf\n", lst, mst);
  if (fabs(lst-mst)<1e-5) check&=1;
  else check &=0;
  printf("check %d\n", check);

  // orbtest
  double     az, el, h, ra, v_az = 119.+18./60.+14.59/3600., v_el = 16./60.+15.88/3600.;
  ra= 20.+40./60.+5.2/3600.;
  dec=-(22.+12./60.);
  gp=38.+55./60.;
  gl=-77.0;
  h=0;
  ut=12.+23./60.;
  dj=JulianDays(1,2,1984);
  orb_alt_az(dj, ut, h, gp, gl, ra, dec, az, el);
  printf("orb_alt_az test az %lf (%lf) el %lf (%lf)\n", az, v_az, el, v_el);

  // SUN AZ/ALT TEST
  /* selftest2:sun_alt_az(julian(3,30,2017),8,1,36, -120, 0);
     [19, 297.1120815341256, 1.714039382226231],  */
  double  dj00;
  //  dj00 = +0.5;
  sun_alt_az(JulianDays(30,3,2017), 19.+8-1, -8., 1., 36., -120., 0., az, el);
  printf("check AZ=%lf ALT=%lf\n", az, el);
  printf("\t272, 3.52\n");
  if (fabs(el-3.52)<1 && fabs(az-272)<1) check&=1;
  else check &= 0;

  //  sprintf(sun_options.location, "%s", las_vegas_location);
  //  sprintf(sun_options.query, "%s", "sunset<now");
  //  printf("DAY\n");
  //  sprintf(sun_options.query, "%s", "sunrise>sunset");

  printf("check %d\n", check);
  /* MOON CHECKS */
/* After so many hardware platforms, and operating systems, I have found it
   necessary to doubt results of all numerical floating point calculations.
   Hence the necessity to benchmark all results with prior values from 
   Peter Duffett-Smith's book page 170: 6000MOON, 6600 MOONRS
*/
  int      isgn, ideg, imin, rise_converges, set_converges;
  double   fsec;
  double   mm, bm, pm, distance_to_moon, phase, solar_longitude, solar_radius,
    v_mm = 260+42/60.+46.34/3600.,
    v_bm = -(0+45/60.+28.16/3600.),
    v_pm = 0+56/60.+3.15/3600.;
  dj = jdays_now;
  dj = JulianDays(25,2,1984);
  ds = 0;
  tzi = 0;
  ut  = 0;
  calculate_moon(dj, ut, mm, bm, pm, distance_to_moon, phase, solar_longitude, solar_radius, true);
  if (fabs(mm-v_mm)<1/3600.&&             // one second accuracy 
      fabs(bm-v_bm)<1/3600.&&
      fabs(pm-v_pm)<1/3600.) check&=1;
  else check&=0;    
  printf("Moon apparent ecliptic mm %lf (%lf) bm %lf (%lf) parallax %lf (%lf) check %d\n",
	 mm, v_mm, bm, v_bm, pm, v_pm, check);
  if (fabs(distance_to_moon-391194.042759)<1&&
      fabs(phase-0.370571)<0.1) check&=1;
  else check&=0;
  printf("earth-moon distance and phase check %d\n", check);

  dj = jdays_now;
  dj = JulianDays(14,8,1989);
  ds = 1;
  tzi = 6;
  ut = 11+42/60.0+20/3600.0-(tzi+ds);
  v_mm = 283+24/60.+1.33/3600.;
  v_bm = -(3+34/60.+4.63/3600.);
  v_pm = 0+57/60.+29.38/3600.;
  calculate_moon(dj, ut, mm, bm, pm, distance_to_moon, phase, solar_longitude, solar_radius, true);
  if (fabs(mm-v_mm)<1/3600.&&             // one second accuracy
      fabs(bm-v_bm)<1/3600.&&
      fabs(pm-v_pm)<1/3600.) check&=1;
  else check&=0;    
  printf("dj %lf ut %lf\n", dj, ut);
  printf("Moon apparent ecliptic mm %lf (%lf) bm %lf (%lf) parallax %lf (%lf) check %d\n",
	 mm, v_mm, bm, v_bm, pm, v_pm, check);
  if (fabs(distance_to_moon-381416.)<1&&
      fabs(phase-0.894)<0.1) check&=1;
  else check&=0;
  printf("earth-moon distance and phase check %d\n", check);

  dj = jdays_now;
  dj = JulianDays(7,1,1984);
  gp = 30.0;
  gl = 0.0;
  ds = 0;
  tzi = 0;
  moon_calculate_rise(utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad, rise_converges, set_converges );
  if (fabs(utu-9.967101)<0.1 && fabs(utd-21.149844)<0.1
      && fabs(au-105.431064)<0.1 && fabs(ad-254.568936)<0.1)
    check&=1;
  else
    check&=0;    
  print_hours("moonrise (9:58:1.564258)", utu+tzi+ds, rise_converges);
  print_hours("moonset (21:8:59.438776)", utd+tzi+ds, set_converges);  
  printf("check %d\n", check);
  dj = JulianDays(11,1,1984);
  moon_calculate_rise(utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad, rise_converges, set_converges );
  printf("utu %lf utd  %lf never_rise  %d never_sets  %d au %lf ad %lf convergence %d %d \n", 
	 utu, utd, never_rise, never_sets, au, ad, rise_converges, set_converges);
  print_hours("moonrise (11:54:54.178575)", utu+tzi+ds, rise_converges);
  print_hours("moonset ()", utd+tzi+ds, set_converges);  
  if (fabs(utu+tzi+ds-(11.+54./60.+54.178575/3600.0))<0.1
      && set_converges==false)
    check&=1;
  else
    check&=0;    

  printf("check %d\n", check);
  dj = JulianDays(1,2,1984);
  gp = 38.92;
  gl = -77.0;
  ds = 0;
  tzi = -5;
  moon_calculate_rise(utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad, rise_converges, set_converges );
  if (fabs(utu-12.390378)<0.1 && fabs(utd-22.153594)<0.1
      && fabs(au-118.082572)<0.1 && fabs(ad-241.917428)<0.1)
    check&=1;
  else
    check&=0;    
  print_hours("moonrise (7:23:25.361480) local time", utu+tzi+ds, rise_converges);
  print_hours("moonset (17:9:12.939055) local time", utd+tzi+ds, set_converges);  

  printf("check %d\n", check);
  // check iteration of moon_calculate_rise_tilnow
  // see if calculation hasn't changed (not verifed that
  // answers are close, but are ok)
  int rise_convergent = 0, set_convergent = 0;
  dj = dj00=42940.500000;
  tzi=-8.000000; 
  ds=1.000000; 
  gl=-114.951226;
  gp=36.048621; 
  jdays_now=42940.552639; 
  now_hour=0; 
  moon_calculate_rise_tilnow(utu, dju, utd, djd, dj, tzi, ds, gl, gp, dj00, now_hour, never_rise, never_sets,
			     au, ad, rise_convergent, set_convergent );

  printf("MOON NOW [RISE utu %lf dju %lf au %lf] [SET utd %lf djd %lf ad %lf] never_rise %d never_sets %d\n",
	 utu, dju, au, utd, djd, ad, never_rise, never_sets);
  if (fabs(utu-15.217902)<.1&&
      fabs(dju-42939.500000)<.1&&
      fabs(utd-3.918381)<.1&&
      fabs(djd- 42939.500000)<.1&&
      never_rise==0 && never_sets==0 &&
      fabs(au-73.992310)<.1&&
      fabs(ad-286.007690)<.1) check &=1;
  else check&=0;

  /* Sun true and apparent ecliptic coordinates */
  double        d2, d3, am,dp,d0;
  double        ecliptic_lat, ecliptic_lon, value_min, value_max;
  double      
    v_solar_lon = 151+0+47.14/3600., 
    v_solar_rad = 1.01099; 
  dj = JulianDays(23, 8, 1984);
  ut = 19+5;
  sun_position(dj, ut, solar_longitude, solar_radius, d2, d3, am); 
  if (fabs(solar_longitude-v_solar_lon)>1/3600.||
      fabs(solar_radius-v_solar_rad)>1e-5) check &=0;
  else check &=1;
  printf("true solar_lon %lf (%lf) solar_radius %lf (%lf) check %d\n", solar_longitude, v_solar_lon, solar_radius, v_solar_rad, check);
  nutation(dj, dp, d0);
  solar_longitude += dp - 5.69e-3;  // degrees
  v_solar_lon = 151+0+12.39/3600.;
  if (fabs(solar_longitude-v_solar_lon)>1/3600.) check &=0;
  else check &=1;
  printf("apparent solar_lon %lf (%lf) check %d\n", solar_longitude, v_solar_lon, check);

  /* Compare against reference aa.usno.navy.mil website
     Las Vegas, Nevada (W115°10'12.0", N36°10'12.0")
     Position of Mars in Night Sky -- due west, 33 degrees up
     Error could be because calculation not taking into consideration
     apparent topocentric position
  */

  dj = JulianDays(27, 7, 2017);
  ut = 0.0;
  tzi = -8.0;
  ds  = 1;
  gl  = -(115+10/60.+12.0/3600.);
  gp = 36+10/60.+12.0/3600.;
  v_az = 271+53/60.+47/3600.;
  v_el = 90-(56+40/60.+28.1/3600.);
  sun_position(dj, 0.0, solar_longitude, solar_radius, d2, d3, am); 
  compute_planet_az_el(MARS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("MARS POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<1 &&                               // why the error?
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  ut = 23.0;
  v_az = 262.+33./60.+44.7/3600.;
  v_el = 90.-(44.+54./60.+44.7/3600.);
  sun_position(dj, 23.0, solar_longitude, solar_radius, d2, d3, am);   
  compute_planet_az_el(MARS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("MARS POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<1 &&
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  ut = 15;   // 14+59/60.;
  v_az = 111.+16./60.+59.8/3600.;
  v_el = 90.-(29.+41./60.+16.8/3600.);
  sun_position(dj, ut, solar_longitude, solar_radius, d2, d3, am);   
  compute_planet_az_el(VENUS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("DATE 27 JULY 2017 (%lf) UT (%lf)\n", dj, ut);
  printf("\tvarying ut hour by minutes varies az-el values by as much as a degree, See venus and mercury\n");
  printf("VENUS UT=15 POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<1 &&
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  ut = 15.0;
  v_az = 83.+12./60.+3.6/3600.;
  v_el = 90.-(63.+57./60.+12.7/3600.);
  compute_planet_az_el(MARS, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("MARS UT=15  POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  printf("\tMARS RA %lf DEC %lf (%lf %lf)\n", ra, dec, 8.+28./60.+49.068/3600., 20+10/60.+57.35/3600);
  printf("\tMARS ECLIPTIC LON %lf LAT %lf\n",  ecliptic_lon, ecliptic_lat);
  if  (fabs(az-v_az)<1 &&
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  ut = 15.0;
  v_az = 251.+58./60.+25.3/3600.;
  v_el = 90.-(78.+21./60.+15.2/3600.);
  compute_planet_az_el(NEPTUNE, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("NEPTUNE  UT=15 POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<1 &&
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  v_az = 77.+1./60.+22.9/3600.;
  v_el = 90.-(90.+34./60.+54.8/3600.);
  compute_planet_az_el(MERCURY, dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon, ra, dec, value_min, value_max, az, el);
  printf("MERCURY  UT=15  POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<1 &&
       fabs(el-v_el)<1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  
  v_az = 65.1;     // REF. www.dailymoonposition.com
  v_el = -27;
  compute_moon_az_el(dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon,
		     ra, dec, value_min, value_max, az, el, phase, distance_to_moon);
  printf("MOON  UT=15  POSITION AZ %lf (%lf) EL %lf (%lf) REF mooncalc.org\n", az, v_az, el, v_el);
  if  (fabs(az-v_az)<3 &&
       fabs(el-v_el)<3) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  v_az = 64+40/60.+9.4/3600.0;
  v_el = 90-(116+53/60.+28.7/3600.);
  printf("\tAZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);  

  // PARALLAX CHECK REF. PETER DUFFETT-SMITH ASTRONOMY WITH YOUR PERSONAL COMPUTER 2nd ed.
  // 2800 PARALLX
  double  lha, a_lha, a_dec, v_lha=21+52/60.+1.58/3600., v_dec=-(8+32/60.+17.4/3600.), ht, hp;
  gl=-100;
  gp=50;
  ht=60;                    // 60m msl
  dj=JulianDays(26,2,1979);
  ut=16+45/60;
  lha=21.+53./60.+25.8/3600.;
  dec=-(7.+41./60.+13./3600.);
  hp=1+1./60.+9./3600.;
  compute_parallax(a_lha, a_dec, lha, dec, gl, gp, dj, ut, ht, hp);
  if (fabs(a_lha-v_lha)<0.00001&&
      fabs(a_dec-v_dec)<0.00001) check &=1;
  else  check &=0;
  printf("Parallax %lf (%lf) %lf (%lf) check %d\n", a_lha, v_lha, a_dec, v_dec, check);
  // See how values change at 1km msl.
  ht=1000.;                    // 60m msl
  compute_parallax(a_lha, a_dec, lha, dec, gl, gp, dj, ut, ht, hp);
  if (fabs(a_lha-v_lha)<0.00001&&
      fabs(a_dec-v_dec)<0.001) check &=1;
  else  check &=0;
  printf("Parallax %lf (%lf) %lf (%lf) 1km msl check %d\n", a_lha, v_lha, a_dec, v_dec, check);

#if 0
  // Jean Meeus Astrongomical Algorthms: correction for parallax
  double 
    eclon = 181+46./60.+22.5/3600., ev_lon,
    eclat = 2+17./60.+26.2/3600., ev_lat,
    hpara =59./60.+27.7/3600.,  
    glat = 50.+5./60.+7.8/3600., obliq = 23.+28./60.+.8/3600., lst = 209.+46./60.+7.9/3600.;
  elliptical_parallax(eclon, eclat, ev_lon, ev_lat, lst, glat, hpara, obliq);
  printf("foobar ecliptic lon %lf (%lf) lat %lf (%lf)\n", ev_lon, eclon, ev_lat, eclat);
#endif
  // Alt-Az Solar Eclipse Day Moon Check REF USNO
  // 3150 S Rocky Comfort Rd, Makanda, IL 62958
  // blueskyvineyard.com
  // (618) 995-9463
  // 37.576748,-89.116411,17
  // Blue Sky Winery
  gl=-89.116411,17;   // -89'6.98  37'34.6
  gp=37.576748;
  ht=0.;                    // 60m msl
  ds=1;
  tzi=6;
  dj=JulianDays(21,8,2017);
  ut = 18.363667;
  v_az = 192.5;
  v_el = 63.8;
  printf("SOLAR ECLIPSE AUGUST 21, 2017 (%lf) %lf %lf,%lf\n", dj, ut, gl, gp);
  compute_moon_az_el(dj, ut, tzi, ds, gl, gp, solar_longitude, ecliptic_lat, ecliptic_lon,
		     ra, dec, value_min, value_max, az, el, phase, distance_to_moon);
  printf("MOON  UT=%lf  POSITION AZ %lf (%lf) EL %lf (%lf) REF USNO\n", ut, az, v_az, el, v_el);
  if  (fabs(az-v_az)<.1 &&
       fabs(el-v_el)<.1) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
  printf("\tAZ %lf (%lf) EL %lf (%lf) REF USNO\n", az, v_az, el, v_el);  

  sun_alt_az(dj, ut, tzi, ds, gp, gl, 0, az, el);
  printf("\tSUN AZ %lf (%lf) EL %lf (%lf) SHOULD MATCH MOON (SOLAR ECLIPSE)\n", az, v_az, el, v_el);  
  if  (fabs(az-v_az)<3 &&
       fabs(el-v_el)<3) check &=1;
  else check &= 0;
  printf("check: %d\n", check);

  // Saros Check REF DUFFETT-SMITH 6800 MOONNF, MEEUS, AND NASA ECLIPSE PAGE
  check &= saros_check();
  printf("saros 145 check completed: %d\n", check);

  // PLANETS REF. PETER DUFFETT-SMITH ASTRONOMY WITH YOUR PERSONAL COMPUTER
  // 4500 PLANS
  // MERCURY
  gl = (2.0+10.4/60.0)/60.0;
  gp = 51.+45./60.;
  ds = 0.0;
  tzi = 0.0;
  dj = JulianDays(21, 12, 1993);
  planet_calculate_rise(MERCURY, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);
  printf("MERCURY RISING UTU %lf AU %lf UTD %lf AD %lf (universal)\n", utu, au, utd, ad);
  if  (fabs(utu-(7+33/60.+49.54/3600.))<0.01666 &&
       fabs(au-(129+45/60.+8.13/3600.))<0.01666 &&
       fabs(utd-(15+9./60.+55.3/3600.))<0.01666 &&
       fabs(ad-(230+14/60.+51.87/3600.))<0.01666) check &=1;
  else check &= 0;
  printf("check: %d\n", check);
 // REFER TO US NAVAL OBSERVATORY (USNO) FOR CALCULATED POSITIONS
  dj = JulianDays(24, 7, 2017);
  gl = -(115.+10./60.+12./3600.);
  gp = 36+10./60.+12./3600.;
  tzi = -8;
  ds  = 1;
  // VENUS 
  //  2017 Jul 24 (Mon)        02:47  63        17:06 297
  planet_calculate_rise(VENUS, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (2+47/60.);
  vtd = (17+6./60.);
  printf("VENUS RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-(63.))<.1 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-(297.)<.1)) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);
  // MARS
  // 2017 Jul 24 (Mon)        05:44  64        12:51 74S        19:58 296
  planet_calculate_rise(MARS, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (5+44/60.);
  vtd = (19+58./60.);
  printf("MARS RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-64.)<.5 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-296.)<.5) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);
  // JUPITER
  // 2017 Jul 24 (Mon)        11:44  96        17:30 49S        23:17 264
  planet_calculate_rise(JUPITER, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (11+44/60.);
  vtd = (23+17./60.);
  printf("JUPITER RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-(96.))<.1 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-(264.)<.1)) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);
  // SATURN
  // 2017 Jul 24 (Mon)        17:00 117        21:54 32S        02:52 243
  planet_calculate_rise(SATURN, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (17);
  vtd = (2+52./60.);
  printf("SATURN RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-(117.))<.1 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-(243.)<.1)) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);
  // URANUS
  // 2017 Jul 24 (Mon)        23:41  77        06:18 64S        12:50 283
  planet_calculate_rise(URANUS, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (23+41/60.);
  vtd = (12+50./60.);
  printf("URANUS RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-(77.))<.5 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-(283.)<.5)) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);
  // NEPTUNE
  // 2017 Jul 24 (Mon)        21:49  99        03:33 47S        09:14 261
  planet_calculate_rise(NEPTUNE, utu, utd, dj, tzi, ds, gl, gp, never_rise, never_sets, au, ad);  
  ltu = POSITIVE_HOURS(utu+tzi+ds);
  ltd = POSITIVE_HOURS(utd+tzi+ds);
  vtu = (21+49/60.);
  vtd = (9+14./60.);
  printf("NEPTUNE RISING UTU %lf (%lf) AU %lf UTD %lf (%lf) AD %lf (local)\n", ltu, vtu, au, ltd, vtd, ad);
  if  (fabs(ltu-vtu)<.1 &&
       fabs(au-99.)<.7 &&
       fabs(ltd-vtd)<.1 &&
       fabs(ad-261.)<.7) check &=1;
  else check &= 0;
  printf("USNO check: %d\n", check);

  // Reference Duffeett-Smith 6800 Moonnf
  extended event_day, event_fraction, argument_latitude,  hours_ut;
  dj = newmoon(1, 9, 1984, 1, argument_latitude, event_day, event_fraction);
  CalendarDays(event_day+event_fraction, now_day, now_month, now_year, now_dayOfWeek);
  hours_ut = (event_fraction-0.5)*24;
  printf("Newmoon AM check %lf %lf %lf : %lf UT=%lf\n", now_day, now_month, now_year, event_fraction, hours_ut);
  if (fabs(now_day-10)<0.1&&fabs(now_month-9)<0.1&&fabs(now_year-1984)<0.1
      &&fabs(hours_ut-7.)<.1) check&=1;
  else check&=0;

  // SOLAR ECLIPSE CHECKS
  double    max_ut_hours, v_max_ut_hours, probabel;
  printf("Solar Eclipses ref: Nasa Great American Eclipse Aug 21, 2017\n");
  // Reference NASA Web Site
  gl = -87.6709;
  gp = 36.9664;
  ht = 0;
  v_max_ut_hours = 18+25/60.+32./3600.;
  solar_eclipse(gl, gp, ht, 21, 8, 2017, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tMaximum %lf (%lf) check %d\n", max_ut_hours, v_max_ut_hours, check);

  // Reference NASA Web Site
  gl = -89.1108;
  gp = 37.5763;
  ht = 0;
  v_max_ut_hours = 18+21/60.+49./3600.;
  solar_eclipse(gl, gp, ht, 21, 8, 2017, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tMaximum %lf (%lf) check %d\n", max_ut_hours, v_max_ut_hours, check);

  // Lat.: 45.0759° N
  // Long.: 24.2983° E
  // nax 11:03:05.4
  gl = 45.0759;
  gp = 24.2983;
  ht = 0;
  v_max_ut_hours = 11+3/60.+5.4/3600.;
  solar_eclipse(gl, gp, ht, 11, 8, 1999, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);

  printf("7000 Eclipse Peter Duffett-Smith\n");
  // Lat. 50   From Peter Duffett-Smith
  // Lon.-5 45' 
  // max 10 11 UT
  gl = -(5+45/60.0);
  gp = 50.;
  ht = 0;
  v_max_ut_hours = 10+11./60.;
  solar_eclipse(gl, gp, ht, 11, 8, 1999, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);
  gl = 18.;
  gp = 42+30./60.;
  ht = 0;
  v_max_ut_hours = 10+53./60.;
  solar_eclipse(gl, gp, ht, 11, 8, 1999, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);
  printf("End 7000 Eclipse Peter Duffett-Smith\n");

  // Lat.: 69.9914° N
  //    Long.: 17.2352° W
  //    1945/07/09 13:27:18.4
  gl = -17.2352;
  gp = 69.9914;
  ht = 0;
  v_max_ut_hours = 13+27/60.+18.4/3600.;
  solar_eclipse(gl, gp, ht, 9, 7, 1945, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);

  //Lat.: 35.5703° S
  // Long.: 47.3693° W
  // 2594/08/07   14:31:07.6
  gl = -47.3693;
  gp = -35.5703;
  ht = 0;
  v_max_ut_hours = 14+31/60.+7.6/3600.;
  solar_eclipse(gl, gp, ht, 7, 8, 2594, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);

  // Lat.: 61.6935° N
  // Long.: 119.555° W
  // 1963/07/20  20:35:38.4
  gl = -119.555;
  gp = 61.6935;
  ht = 0;
  v_max_ut_hours = 20+35/60.+38.4/3600.;
  solar_eclipse(gl, gp, ht, 20, 7, 1963, probabel, max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;
  printf("\tProbabel %lf Maximum %lf (%lf) check %d\n", probabel, max_ut_hours, v_max_ut_hours, check);



  if (check == 1)
    printf("PASSED SELFTEST ( RESULT CODE= %d)\n", check);
  else
    printf("FAILED SELFTEST ( RESULT CODE= %d)\n", check);

  // ECLIPSE AUGUST 21, 2017 REF NASA WEBSITE
  // Lat.: 37.5763° N
  // Long.: 89.1108° W
  // Total Solar Eclipse
  // Duration of Totality: 2m40.2s
  // Magnitude: 1.015 
  // Obscuration: 100.00% 
  // EventDateTime (UT)AltAzi 
  // Start of partial eclipse (C1) : 2017/08/21 16:52:44.0 60.3°145.5°
  // Start of total eclipse (C2) : 2017/08/21 18:20:29.0 63.9°191.7° 
  // Maximum eclipse : 2017/08/21 18:21:49.2  63.8°192.5° 
  // End of total eclipse (C3) : 2017/08/21 18:23:09.2  63.7°193.2° 
  // End of partial eclipse (C4) : 2017/08/21 19:47:54.1 54.6°230.4°
  const int fullmoon = 0;  // 0 signifies new moon
  extended eclipse_certain, v_hours_ut;
  extern char			*monthStringArray[];
  extern char			*dayStringArray[];
  printf("ECLIPSE AUGUST 21, 2017\n");
  newmoon(21, 8, 2017, fullmoon, argument_latitude, event_day, event_fraction);
  CalendarDays(event_day, now_day, now_month, now_year, now_dayOfWeek);
  printf("NEW MOON (8/21/2017) %lf %d %s %d %s\n", event_day, 
	 (int)now_day, monthStringArray[(int)now_month-1], (int)now_year, dayStringArray[(int)now_dayOfWeek-1]);
  hours_ut = (event_fraction-0.5)*24;
  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;
  v_hours_ut = 18+21/60.+49.2/3600.;
  //  minutes_ut = (hours_ut - floor(hours_ut))*60;
  //  sec_ut = (minutes_ut - floor(minutes_ut))*60;
  printf("\tTime: %lf (%lf) Argument of Latitude %lf\n", hours_ut, v_hours_ut, argument_latitude);
  eclipse_check(argument_latitude, eclipse_certain);
  printf("\tEclipse probability %lf\n", eclipse_certain);

  // Astronomical Formulae for Calculators Jean Meeus, Solar eclipse
  newmoon(2, 10, 1978, fullmoon, argument_latitude, event_day, event_fraction);
  printf("Oct 2, 1978 argument of latitude %lf (%lf) degrees\n",
	 argument_latitude*180./M_PI, 14.3687);
  newmoon(16, 2, 1980, fullmoon, argument_latitude, event_day, event_fraction);
  printf("Feb 16 1980 argument of latitude %lf (%lf) degrees\n",
	 argument_latitude*180./M_PI, 175.7671);
  printf("Reference: Astronomical Formulae for Calculators Jean Meeus\n");
  // CHECKS SAROS145
  // 2179 Nov 28 
  newmoon(18, 11, 2179, fullmoon, argument_latitude, event_day, event_fraction);
  CalendarDays(event_day, now_day, now_month, now_year, now_dayOfWeek);
  printf("NEW MOON %lf %d %s %d %s\n", event_day, 
	 (int)now_day, monthStringArray[(int)now_month-1], (int)now_year, dayStringArray[(int)now_dayOfWeek-1]);
  hours_ut = (event_fraction-0.5)*24;
  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;
  v_hours_ut = 18+54/60.+18./3600.;
  printf("\tTime: %lf (%lf) Argument of Latitude %lf\n", hours_ut, v_hours_ut, argument_latitude);
  eclipse_check(argument_latitude, eclipse_certain);
  printf("\tEclipse probability %lf\n", eclipse_certain);
  //  2450 May 12  02:29:44  
  newmoon(12, 5, 2450, fullmoon, argument_latitude, event_day, event_fraction);
  CalendarDays(event_day, now_day, now_month, now_year, now_dayOfWeek);
  printf("NEW MOON %lf %d %s %d %s\n", event_day, 
	 (int)now_day, monthStringArray[(int)now_month-1], (int)now_year, dayStringArray[(int)now_dayOfWeek-1]);
  hours_ut = (event_fraction-0.5)*24;
  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;
  v_hours_ut = 2+29/60.+44./3600.;
  printf("\tTime: %lf (%lf) Argument of Latitude %lf\n", hours_ut, v_hours_ut, argument_latitude);
  eclipse_check(argument_latitude, eclipse_certain);
  printf("\tEclipse probability %lf\n", eclipse_certain);

#if 1
  printf("nicks eclipse\n");
  // Lick Creek IL  37.5226° N, 89.0751° W
  gl = -89.0751;
  gp = 37.5226;
  // Reference NASA Web Site
  gl = -87.6709;
  gp = 36.9664;
  ht = 0;
  v_max_ut_hours = 18+25/60.+32./3600.;
  solar_eclipse(gl, gp, ht, 21, 8, 2017, probabel, max_ut_hours);
  // printf("Start of total eclipse %lf\n",
  // 	 18+24/60.+11./3600.);
  // printf("\tMaximum %lf\n", 
  // 	 18+25/60.+32./3600.);
  // printf("\tEnd of total eclipse %lf\n",
  // 	 18+26/60.+51/3600.);
  printf("\tMaximum %lf (%lf)\n", max_ut_hours, v_max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;

  // Reference NASA Web Site
  gl = -89.1108;
  gp = 37.5763;
  ht = 0;
  v_max_ut_hours = 18+21/60.+49./3600.;
  solar_eclipse(gl, gp, ht, 21, 8, 2017, probabel, max_ut_hours);
  // printf("Start of total eclipse %lf\n",
  // 	 18+20/60.+29/3600.);
  // printf("\tMaximum %lf\n", 
  // 	 18+21/60.+49./3600.);
  // printf("\tEnd of total eclipse %lf\n",
  // 	 18+23/60.+9/3600.);
  // printf("\tMaximum %lf (%lf)\n", max_ut_hours, v_max_ut_hours);
  if (fabs(max_ut_hours-v_max_ut_hours)>1./60.) check &= 0;
  else check &= 1;

#endif

  // NEXT MONTH
  printf("NEXT MONTH\n");
  dj = newmoon(21, 9, 2017, fullmoon, argument_latitude, event_day, event_fraction);
  hours_ut = (event_fraction-0.5)*24;
  if (hours_ut>24) hours_ut -= 24;
  if (hours_ut<0) hours_ut += 24;
  v_hours_ut = 5+29/60.;
  CalendarDays(dj, now_day, now_month, now_year, now_dayOfWeek);
  printf("NEXT NEW MOON %lf %d %s %d %s\n", event_day, 
	 (int)now_day, monthStringArray[(int)now_month-1], (int)now_year, dayStringArray[(int)now_dayOfWeek-1]);
  printf("\tTime: %lf (%lf) Argument of Latitude %lf\n", hours_ut, v_hours_ut, argument_latitude);
  eclipse_check(argument_latitude, eclipse_certain);
  printf("\tEclipse probability %lf\n", eclipse_certain);


  printf("**************************************\n");
  printf("NIGHT\n");
  sprintf(sun_options.query, "%s", "sunrise<sunset");
  check = logic_withcheck(sun_options.query);
  printf("QUERY %s result %d\n", sun_options.query, check);


  printf("DAY\n");
  sprintf(sun_options.query, "%s", "sunrise>sunset");
  check = logic_withcheck(sun_options.query);
  printf("QUERY %s result %d\n", sun_options.query, check);

  printf("MOONNIGHT\n");
  sprintf(sun_options.query, "%s", "moonrise<moonset");
  check = logic_withcheck(sun_options.query);
  printf("QUERY %s result %d\n", sun_options.query, check);


  printf("MOONDAY\n");
  sprintf(sun_options.query, "%s", "moonrise>moonset");
  check = logic_withcheck(sun_options.query);
  printf("QUERY %s result %d\n", sun_options.query, check);
}

struct saros_table {
  int d, m, y;                  // day, month, year
  extended argument_latitude;   // radians
  extended probability;         // 0..1
};

int
saros_check()
{
  int i, success = true;
  const int fullmoon = 0;
  extended d, m, y, argument_latitude, event_day, event_fraction, probability;
  extended v_argument_latitude, v_probability;
  saros_table
    *an_event,
    saros_table_145[77] = {{4,1,1639,0.2936129495987615,0.5},
			 {14,1,1657,0.285394388349568,0.5},
			 {25,1,1675,0.2771739318932241,0.5},
			 {5,2,1693,0.2689515816937373,0.5},
			 {17,2,1711,0.2607273392236868,0.5},
			 {27,2,1729,0.25250120594708,0.5},
			 {11,3,1747,0.2442731833279243,0.5},
			 {21,3,1765,0.2360432728387979,1.0},
			 {1,4,1783,0.2278114759422796,1.0},
			 {13,4,1801,0.2195777941080923,1.0},
			 {24,4,1819,0.2113422288002424,1.0},
			 {4,5,1837,0.2031047814877376,1.0},
			 {16,5,1855,0.1948654536367282,1.0},
			 {26,5,1873,0.1866242467137217,1.0},
			 {6,6,1891,0.1783811621861175,1.0},
			 {17,6,1909,0.1701362015205135,1.0},
			 {29,6,1927,0.1618893661838626,1.0},
			 {9,7,1945,0.1536406576429401,1.0},
			 {20,7,1963,0.1453900773650573,1.0},
			 {31,7,1981,0.1371376268160072,1.0},
			 {11,8,1999,0.1288833074647966,1.0},
			 {21,8,2017,0.1206271207754348,1.0},
			 {2,9,2035,0.1123690682162122,1.0},
			 {12,9,2053,0.104109151255424,1.0},
			 {23,9,2071,0.09584737135850627,1.0},
			 {4,10,2089,0.08758372998946444,1.0},
			 {16,10,2107,0.07931822862087872,1.0},
			 {26,10,2125,0.07105086871675648,1.0},
			 {7,11,2143,0.06278165174110417,1.0},
			 {17,11,2161,0.05451057916650197,1.0},
			 {28,11,2179,0.04623765245695631,1.0},
			 {9,12,2197,0.03796287307647557,1.0},
			 {21,12,2215,0.02968624249763697,1.0},
			 {31,12,2233,0.02140776218159166,1.0},
			 {12,1,2252,0.01312743360377707,1.0},
			 {22,1,2270,0.004845258219625876,1.0},
			 {2,2,2288,6.279746544681306,1.0},
			 {14,2,2306,6.271460680102223,1.0},
			 {25,2,2324,6.263172973117398,1.0},
			 {8,3,2342,6.25488342519941,1.0},
			 {18,3,2360,6.24659203782084,1.0},
			 {29,3,2378,6.238298812437122,1.0},
			 {9,4,2396,6.230003750520836,1.0},
			 {20,4,2414,6.221706853544558,1.0},
			 {30,4,2432,6.213408122963727,1.0},
			 {12,5,2450,6.205107560250921,1.0},
			 {22,5,2468,6.196805166878719,1.0},
			 {2,6,2486,6.188500944302556,1.0},
			 {14,6,2504,6.180194893995012,1.0},
			 {25,6,2522,6.171887017428665,1.0},
			 {5,7,2540,6.163577316058951,1.0},
			 {17,7,2558,6.155265791358449,1.0},
			 {27,7,2576,6.146952444794024,1.0},
			 {7,8,2594,6.138637277843968,1.0},
			 {18,8,2612,6.13032029195229,1.0},
			 {30,8,2630,6.122001488597281,1.0},
			 {9,9,2648,6.113680869245807,1.0},
			 {20,9,2666,6.105358435364732,1.0},
			 {1,10,2684,6.097034188420922,1.0},
			 {13,10,2702,6.088708129892669,1.0},
			 {23,10,2720,6.080380261223979,1.0},
			 {4,11,2738,6.072050583893146,1.0},
			 {14,11,2756,6.063719099367036,1.0},
			 {25,11,2774,6.055385809112512,1.0},
			 {6,12,2792,6.047050714596438,1.0},
			 {17,12,2810,6.03871381729711,0.5},
			 {28,12,2828,6.030375118658533,0.5},
			 {8,1,2847,6.022034620158999,0.5},
			 {18,1,2865,6.013692323265375,0.5},
			 {30,1,2883,6.005348229444525,0.5},
			 {10,2,2901,5.997002340163314,0.5},
			 {21,2,2919,5.988654656900033,0.5},
			 {4,3,2937,5.980305181098691,0.5},
			 {15,3,2955,5.971953914237581,0.5},
			 {25,3,2973,5.963600857783566,0.5},
			 {6,4,2991,5.955246013203513,0.5},
			 {17,4,3009,5.946889381964284,0.5}};
  an_event = &saros_table_145[21];
  for (i=0;i<77;i++){
    an_event = &saros_table_145[i];
    d = an_event->d, m = an_event->m, y = an_event->y;
    v_argument_latitude=an_event->argument_latitude;
    v_probability=an_event->probability;
    newmoon(d, m, y, fullmoon, argument_latitude, event_day, event_fraction);
    eclipse_check(argument_latitude, probability);
    if (fabs(argument_latitude-v_argument_latitude)>0.0001
	|| fabs(probability-v_probability)>0.001) success &= 0;
  }
  return success;
}
